/*
	Printing.c
	
	This file contains the printing routines for "all shapes".

	Created		9/2/92		- Dave Hersey

	----------------------------------------------

	2/1/93	debugged and plopped on GX CD.  - dmh
	9/93	updated to run with the 2 "GXified" interface files - PLA
	12/93	updated for b3.  - dmh
*/

#include "font library.h"
#include "graphics libraries.h"
#include "graphics toolbox.h"
#include "graphics debugging.h"
#include "qd library.h"
#include "PrintingManager.h"
#include "PrintingResTypes.h"
#include <PrintingMessages.h>
#include <Collections.h>
#include <stdio.h>

#include "Banana Jr.h"


/*------ SetUpEditMenuRec ------------------------------------------------------------------------------------*/
//
//	This routine sets up an gxEditMenuRecord which references our edit menu.  This structure
//	is used by the GXJobDefaultFormatDialog and GXJobPrintDialog calls to allow cut, copy,
//	& paste operations from the dialogs.
//
void SetUpEditMenuRec(gxEditMenuRecord *edMenuRec)
{

	edMenuRec->editMenuID = mEdit;
	edMenuRec->cutItem = iCut;
	edMenuRec->copyItem = iCopy;
	edMenuRec->pasteItem = iPaste;
	edMenuRec->clearItem = iClear;
	edMenuRec->undoItem = iUndo;
}


/*------ DoDocFormat ---------------------------------------------------------------------------------*/
//
//	This routine performs GX's equivalent of the Page Setup (PrStlDialog) call of
//	the old printing architecture.
//
OSErr DoDocFormat(WindowPtr wind, gxDialogResult	*result)
{
	OSErr			err;
	gxEditMenuRecord	edMenuRec;
	gxJob				docJob;
	TH_Doc			doc;

//	If we have a non-nil WindowPtr, set up our edit menu record and handle the job
//	format dialog.  Remember to check for errors.

	if (wind)
	{
		doc = GetDoc(wind);
		docJob = GetDocJob(doc);
		SetUpEditMenuRec(&edMenuRec);

		*result = GXJobDefaultFormatDialog(docJob, &edMenuRec);
		err = GXGetJobError(docJob);
	}
	
	return err;
}


/*------ DoPageFormat ------------------------------------------------------------------------------*/
//
//	This routine performs a by-page format.  In other words, it sets formatting info
//	for only the current page.
//
OSErr DoPageFormat(WindowPtr wind, gxDialogResult *result)
{
	OSErr				err = noErr;
	gxEditMenuRecord	edMenuRec;
	gxJob				docJob;
	TH_Doc				doc;
	TH_Page				docPage;
	gxFormat			pageFormat;

//	If we have a non-nil WindowPtr, set up our edit menu record and handle the job
//	format dialog.  Remember to check for errors.

	if (wind)
	{
		SetUpEditMenuRec(&edMenuRec);
		doc = GetDoc(wind);
		docJob = GetDocJob(doc);
		docPage = GetDocPage(doc, (*doc)->curPage);
		pageFormat = (*docPage)->pageFormat;

		*result = GXFormatDialog(pageFormat, &edMenuRec, nil);
		
		if (*result == gxRevertSelected)
		{
			GXDisposeFormat(pageFormat);
			(*docPage)->pageFormat = GXNewFormat(docJob);
		}
		
		err = GXGetJobError(docJob);
	}
	
	return err;
}


/*------ DoPrinting ----------------------------------------------------------------------------------*/
//
//	This routine performs GX's equivalent of the PrJobDialog call in the old
// printing architecture, and then prints the document if the user wants to.
//
OSErr DoPrinting(WindowPtr wind)
{
	OSErr			err = noErr;
	gxDialogResult	result;
	gxEditMenuRecord	edMenuRec;
	gxJob				docJob;
	TH_Doc			doc;
	gxFormat			jobFormat;
	gxMapping			formatMapping;
	gxViewPort		formatVPort;

//	If we have a non-nil WindowPtr, set up our edit menu record and handle the print
//	job dialog.  Remember to check for errors.  If no errors occur, and the user clicks
//  ok, print the window's document.

	if (wind)
	{
		doc = GetDoc(wind);
		docJob = GetDocJob(doc);
		
		SetUpEditMenuRec(&edMenuRec);
		result = GXJobPrintDialog(docJob, &edMenuRec);
		err = GXGetJobError(docJob);
				
		if ((result == gxOKSelected) && !(err))
			err = DoPrintOne(wind);
	}
	
	return err;
}



/*------ DoPrintOne ----------------------------------------------------------------------------------*/
//
//	This routine prints one copy of the window's document using whatever job and format
//	is currently attached to it.  (If the window wwere just created and then the user
//	selected Print One w/o first selecting Page Setup, the system default job and
//	format are stored with this document.
//
OSErr DoPrintOne(WindowPtr wind)
{
	OSErr		err = noErr;
	gxJob		docJob;
	Str255		title;
	TH_Doc		doc;
	TH_Page		docPage;
	gxFormat	pageFormat;
	gxShape 	pageShape, coverPage;
	long		firstPg, lastPg, pg, numPages;

//	If we have a non-nil WindowPtr, start the print job, print a page and then finish
//	the job.  Remember to check those errors!

	if (wind)
	{
		doc = GetDoc(wind);
		docJob = GetDocJob(doc);
		numPages = (*doc)->numPages;
		GetWTitle(wind, title);
		
		GXGetJobPageRange(docJob, &firstPg, &lastPg);

		firstPg = (firstPg < 1)? 1: (firstPg > numPages)? numPages: firstPg;
		lastPg = (lastPg < firstPg)? firstPg: (lastPg > numPages)? numPages: lastPg;
		numPages = lastPg - firstPg +1;
		
		GXStartJob(docJob, title, numPages);
		err = GXGetJobError(docJob);
		
		for (pg = firstPg; (!err) && (pg <= lastPg); pg++)
		{
			docPage = GetDocPage(doc, pg);
			pageFormat = (*docPage)->pageFormat;
			pageShape = GXCopyToShape(nil, GetDocShape(doc, pg));
			CheckForIdiots(pageFormat, pageShape);

			GXPrintPage(docJob, pg, pageFormat, pageShape);
			
			GXDisposeShape(pageShape);
			err = GXGetJobError(docJob);
		}

		GXFinishJob(docJob);
		if (!err) err = GXGetJobError(docJob);
	}
	return err;
}


void CheckForIdiots(gxFormat aFormat, gxShape pageShape)
{
	OSErr					err;
	Collection				fmtCollection;
	T_BananaCollection		bananaConfig;
	gxShape 					theText;
	Str255					textStr;
	long					numChars;
	
	fmtCollection = GXGetFormatCollection(aFormat);

	err = GetCollectionItem(fmtCollection, kBananaCollectionType,
					  		gxPrintingTagID, nil, &bananaConfig);
	
	if (!err)
	{
		if (bananaConfig.one || bananaConfig.two ||
			bananaConfig.three || bananaConfig.four)
		{
			theText = GXNewShape(gxTextType);
			GXSetShapeTextSize(theText, ff(32));
			SetShapeCommonFont(theText, timesFont);
			GXMoveShapeTo(theText, ff(10), ff(40));

			if (bananaConfig.two)	// Idiots setting
			{
				numChars = sprintf((Ptr) &textStr[0], "%s", "Idiots enabled.");
				GXSetText(theText, numChars, textStr, nil);

				AddToShape(pageShape, theText);
				GXMoveShape(theText, ff(0), ff(40));
			}

			if (bananaConfig.three)	// Random Printing setting
			{
				numChars = sprintf((Ptr) &textStr[0], "%s", "Random Printing Enabled.");
				GXSetText(theText, numChars, textStr, nil);

				AddToShape(pageShape, theText);
				GXMoveShape(theText, ff(0), ff(40));
			}

			if (bananaConfig.four)	// Page Shuffling setting
			{
				numChars = sprintf((Ptr) &textStr[0], "%s", "Page Shuffling enabled.");
				GXSetText(theText, numChars, textStr, nil);

				AddToShape(pageShape, theText);
				GXMoveShape(theText, ff(0), ff(40));
			}

			if (bananaConfig.one)	// 360 Rotation setting
			{
				numChars = sprintf((Ptr) &textStr[0], "%s", "360 Rotation enabled.");
				GXSetText(theText, numChars, textStr, nil);

				AddToShape(pageShape, theText);
			}

			GXDisposeShape(theText);
		}
	}
}






