/*________________________________________________________

	File: DTP Menu Alert.c

	C code for a printing extension that adds an alert
	item to the desktop printer menu.

	Dave Hersey
	Apple Developer Technical Support

	12/01/92 	- dmh - created.
	 4/26/93	- dmh - updated to use recommended approach
	 					to global data initialization.
	12/22/93 	- dmh - updated for b3.

	(Note: all functions are in the Mark menu.)
	
__________________________________________________________*/

#include <Types.h>
#include <Errors.h>
#include <Resources.h>
#include <ToolUtils.h>
#include <Collections.h>
#include <Messages.h>
#include <PrintingManager.h>
#include <PrintingMessages.h>

#include "DTP Menu Alert.h"


/*******************************************************************
	InitGlobalData is used to initialize any global data we need to
	in our initialize message override.  It's critical that you do
	things this way, rather than access the data in the same scope
	that you call NewMessageGlobals.  Otherwise, some compilers
	will give you code that references an invalid a5 world.

********************************************************************/

OSErr InitGlobalData()
{
// Initialize any global data here.
	
	return noErr;
}


/*******************************************************************
	NewInitialize is our override for the Initialize message.  In
	here, you shouldn't initialize anything directly-- call
	InitGlobalData for that.  Once you create the A5 world with
	NewMessageGlobals, you can access your global data just like
	you were an app.  Whenever you're called, your global data will
	be valid.
	
********************************************************************/

OSErr NewInitialize()
{
	OSErr	err;

// Create an A5 world, and initialize our global data.

	err = NewMessageGlobals(A5Size(), A5Init);
	
	if (!err) err = InitGlobalData();
	
	return err;
}


/*******************************************************************
	NewShutDown is our override for the ShutDown message.  We
	simply throw away our a5 world.
	
********************************************************************/

OSErr NewShutDown()
{
	DisposeMessageGlobals();
	return noErr;
}


/*******************************************************************
	NewGetDTPMenuList is our override for GetDTPMenuList.  All we
	do is add our item to the DTP printer menu and forward the
	message.
	
********************************************************************/

OSErr NewGetDTPMenuList(MenuHandle aMenu)
{
	OSErr			err1, err2;
	StringHandle	itemStr;
	
	itemStr = (StringHandle) GetString(r_AlertString);
	err1 = ResError();
	
	if (err1 == noErr)
	{
		HLock((Handle) itemStr);
		AppendMenu(aMenu, (ConstStr255Param) *itemStr);
		ReleaseResource((Handle) itemStr);
	}

	err2 = Forward_GXGetDTPMenuList(aMenu);
	
	if (err1 == noErr) err1 = err2;

	return err1;
}


/*******************************************************************
	NewDTPMenuSelect is our override for DTPMenuSelect.  If our
	DTP printer menu item is selected, we just put up an alert.
	
********************************************************************/

OSErr NewDTPMenuSelect(short ID, Boolean *done)
{
	OSErr			err = noErr;
	gxStatusRecord	*pStatus;

	switch (ID)
	{
		case kAlertItem:

			pStatus = (gxStatusRecord *) NewPtr( sizeof(gxStatusRecord) + sizeof(short) );

			if (pStatus == nil) 
			{
				err = MemError();
				goto CantMakeStatusRecord;
			}
			
	// Fill in the status record appropriately.

			pStatus->statResId		= kUserAttentionID;
			pStatus->statResIndex	= kAlertStatusIdx;
			pStatus->dialogResult	= nil;
			pStatus->bufferLen		= nil;

	// Continue alerting the user until he responds.

			do
			{
				err = GXAlertTheUser(pStatus);
			}
			while ((err == noErr) && (pStatus->dialogResult == nil));

			*done = true;
			DisposPtr((Ptr) pStatus);
			break;

		default:
			err = Forward_GXDTPMenuSelect(ID -kNumMenuItemsWeAdded);
	}

CantMakeStatusRecord:

	return err;
}


/*******************************************************************
	NewFetchTaggedData is an override for GXFetchTaggedData.  If
	you're going to put up an alert using GXAlertTheUser, you must
	override this message like so.
	
********************************************************************/

OSErr NewFetchTaggedData(ResType theType, short theID, Handle *theHandle,
						 OSType owner)
{
	OSErr	err = noErr;

	if (owner == kCreator)					// Is it for us?
	{
		Handle	loadedHandle;				// Yes, get and detach.
		
		loadedHandle = GetResource(theType, theID);	

		if (loadedHandle)
		{
			DetachResource(loadedHandle);
			*theHandle = loadedHandle;
		}
		else	// error -- let the system try.
			err = Forward_GXFetchTaggedData(theType, theID, theHandle, owner);
	}
	else									// No, forward it.
		err = Forward_GXFetchTaggedData(theType, theID, theHandle, owner);

	return err;
}