/*
	Adjust Layout Sample.c
	
	This file contains the calls that an application needs to make the B3 QuickDraw GX shell
	work correctly.
	
	1992 - 1993  Apple Computer, Inc.
	All rights reserved.
*/

#include <StdIO.h>

#include <resources.h>
#include <events.h>
#include <memory.h>
#include <windows.h>
#include <toolutils.h>

#include "font library.h"
#include "graphics libraries.h"
#include "graphics toolbox.h"
#include "graphics errors.h"
#include "graphics debugging.h"
#include "qd library.h"
#include "PrintingMessages.h"
#include "PrintingManager.h"

#include "B3 QuickDraw GX shell.h"
#include "Adjust Layout Sample.h"


/*------ Global variables for the shell ----------------------------------------------------*/

// Set up the title and size of the window

Str255 		gWindowTitle;
Rect 		gWindowQDRect;

// If gDebugging = TRUE, graphics library errors and notices will be posted.  This
// functionality will only work with the "debugging" version of QuickDraw GX.
// If the debugging version is not installed, nothing bad will happen, but these
// functions will not work. 

Boolean		gDebugging = true;

// Set  "gGiveMeValidation" to TRUE if you want receive run-time validation.

Boolean		gGiveMeValidation = true;


// gGraphicsHeapSize sets the size of the graphics heap created by calling the
// GXNewGraphicsClient routine in main () within B3 QuickDraw GX shell.c.  You can determine
// the amount of graphics heap required by using GraphicsBug.  I'm giving it 600K,
// since we need abunch if we have several windows open.

long		gGraphicsHeapSize = 600;


/*------ DoSetup ---------------------------------------------------------------------------*/
//
//	Here's where we initialize any global variables our application needs.  We have two of
//	them -- gWindowTitle and gWindowQDRect.  We obtain both of them from our resource fork,
//	so we don't have to recompile to change such things for international markets, etc.

void DoSetup(void)
{

	Handle	windRectHandle;

// Get the title and rectangle for our window.  These are in our resource fork -- the
// title as a string in 'STR#' resource number 'rWindTitleString' and the rectangle as
// 'RECT' resource number 'rWindRectangle'.  These are all defined in the header file
// "B3 QuickDraw GX shell.h".

	GetIndString(gWindowTitle, rWindTitleString, rWindTitleStringIndex);
	windRectHandle = GetResource('RECT',rWindRectangle);
	gWindowQDRect = *(RectPtr)*windRectHandle;
	ReleaseResource(windRectHandle);
}

/*------ DoInitialization ------------------------------------------------------------------*/
//
//	In this function we create and initialize the the private document structure for
//  a new window.  This structure contains the print job and the shape which is drawn
//  in the window.  We store this data in a handle and hang it off the window's refCon
//  field for easy retrieval.  By doing this, rather than using globals, we can create
//  many windows containing unique print jobs and shapes. 

OSErr DoInitialization(wind)
WindowPtr wind;
{
	OSErr	err = noErr;
	gxJob	docJob;
	gxShape	docPage;
	TH_Doc	windDoc;


// Create the page shape. We set the unique items attribute to make sure that each item
// added to the picture has a unique reference. If this attribute was not set, we would
// not see all copies of anything we add to the shape multiple times -- we'd just see
// the last version added.		

	docPage = GXNewShape(gxPictureType);
	GXSetShapeAttributes(docPage, gxUniqueItemsShape);
	
	
// Create a print job for this document.  This will be the same as the system default until
// the user goes through the dialogs for Page Setup or Print

	err = GXNewJob(&docJob);
	
	
// If there are no errors, create a handle the size of our document structure and store
// the print job and page shape in it.  Store the handle in the window's refCon field so
// that we can get at it.  (Note that the utility routines "GetDocJob" and "GetDocShape"
// can be used to do this easily.

	if (!err)
	{
		windDoc = (TH_Doc) NewHandleClear(sizeof(T_Doc));

		if (!windDoc)
			err = MemError();
		else
		{
			(*windDoc)->docJob = docJob;
			(*windDoc)->docPage = docPage;
			((WindowPeek) wind)->refCon = (long) windDoc;
		}

// Now install our application override for PrintingEvent so that we can
// support the new movable-modal printing dialog boxes.

		GXInstallApplicationOverride(docJob, gxPrintingEvent, MyPrintingEventOverride);

	}

	return err;
}


/*------ DoDraw ----------------------------------------------------------------------------*/
//
//	Draw the contents of the window.
// 
void DoDraw(wind)
WindowPtr wind;
{
	 GXDrawShape (GetDocShape(wind));
}


/*------ StrLength ------------------------------------------------------------------------*/
//
//	This function takes the length of a C-style string quickly without requiring an ANSI
//  library to be linked in.
//
static short StrLength(char *s)
{
	short len;

	for (len = 0; *s++ != 0; len++) ;
	return len;
}



/*------ CreateSampleLayoutImage -----------------------------------------------------------*/
//
//	This function creates  primitive shapes and adds them to the window's page shape.
//
void CreateSampleLayoutImage(WindowPtr wind)
{
	gxShape layout;									// the layout shape we're building
	gxShape thePage;								// the picture shape for our window
	gxRunControls controls;							// run controls for the layout
	gxLayoutOptions layoutOptions;					// options for our layout
	StyleRunOverrides overrides;					// structure defined in layout library.h
													// and used in several library calls to
													// make working with style runs easier
	gxStyle glyphStyles[3];							// array of three different styles for
													// glyphs we use
	// Here's the text of the three runs
	
	char *text1 = "Office";
	char *text2 = "AWAY AND SOME ";
	char *text3 = "JUNK.";
	
	char *textRuns[3];								// and the array to hold them
	
	short level0 = 0;								// We have to pass levels by address,
													// so here's a variable to hold one			
	short textLengths[3];							// array of text lengths for GXNewLayout
	short totalLength;								// sum of the text lengths
	
// The "glyphX" values are #define'd in Adjust Layout Sample.h.  There are four
// values in each set -- the glyph code of the first glyph, the glyph code of the
// second glyph, how much with-stream kerning to apply and how much cross-stream
// kerning to apply.  We're kerning with the pairs A-W, A-Y and W-A here.
// Normally you would _not_ hard-code glyph codes because they can change from font
// to font, but this is a demonstration, not a shipping application.  Try this on
// some non-Roman fonts, or some where the glyph codes for the ASCII characters aren't
// the same as their ASCII value, and you'll see strange results.
	
	gxKerningAdjustment adjustments[nAdjustments] = 
	{{glyphA, glyphW, withFactor, crossFactor}, 
	 {glyphA, glyphY, withFactor, crossFactor}, 
	 {glyphW, glyphA, withFactor, crossFactor}};
	 
// The gxGlyphSubstitution array lets us substitute one glyph for another by
// glyph code.  Again taking the liberty of assuming glyph codes for Times Roman,
// this set substitutes the second glyph in each pair for the first when the layout
// shape is drawn.
	 
	gxGlyphSubstitution substitutions[nSubstitutions] = 
	{{glyphJ, glyphW},
	 {glyphU, glyphA},
	 {glyphN, glyphV},
	 {glyphK, glyphE}};
	
	gxPoint posn;									// The position for the layout shape

	
// OK, now to work.  First, initialize the textRuns array.

	textRuns[0] = text1;
	textRuns[1] = text2;
	textRuns[2] = text3;
	
	
// Next, initialize the textLengths array.  StrLength is in this file to avoid linking
// in a huge ANSI library.
	
	textLengths[0] = StrLength (text1);
	textLengths[1] = StrLength (text2);
	textLengths[2] = StrLength (text3);
	
	
// Now add them all up to get the total length.
	
	totalLength = textLengths[0] + textLengths[1] + textLengths[2];
	
	
// Make default gxLayoutOptions, gxRunControls and StyleRunOverrides structures.
// The library routines which initialize these default structures may be found in
// layout library.c.
	
	InitializeLayoutOptions (&layoutOptions);
	InitializeRunControls (&controls);
	InitializeStyleRunOverrides (&overrides);
	
// Position the layout half way down the left edge of the window. Set 
// the layout's width to the window's width and set the flushness to 1/2, this
// will cause the layout to center in the window.

	layoutOptions.width = ff(gWindowQDRect.right - gWindowQDRect.left);
	layoutOptions.flush = fract1/2;		// fract1 is defined in math types.h

	posn.x = 0;
	posn.y = ff((gWindowQDRect.bottom - gWindowQDRect.top) / 2);

	
// Initialize the style runs.  The first one (#0) is 60 point Times Roman.
// We create the style with NewLayoutStyle, found in layout library.c.

	glyphStyles[0] = NewLayoutStyle(
		(char *) "\pTimes Roman",	// The gxFontName for this style
		ff(60),						// the text size in fixed point
		0,							// gxTextAttribute (none for us)
		&controls,					// run controls (none for this style)
		nil,						// run features (none right now)
		0,							// count of run features (no run features == 0)
		nil);						// style run overrides (none right now)

// Now we create the next style.  We reuse the controls and style run overrides
// structures, so now we set those up for the second (#1) style.

// Initialize the run controls:
// Set the cross stream shift to 30 to raise the text 30 points,
// and the with stream shift to -4 to move the text 4 points
// closer together than normal kerning would move it.

	controls.crossStreamShift = ff(30);
	controls.beforeWithStreamShift = ff(-2);
	controls.afterWithStreamShift = ff(-2);

// Now set up the style run overrides:
// Set up the kerning adjustments to the array we already created, and
// put the count in.  nAdjustments is #define'd in Adjust Layout Sample.h.
	
	overrides.kerningAdjustments = adjustments;
	overrides.kerningAdjustmentsCount = nAdjustments;

// We're ready to create the second style -- 30 pt. Times Roman with kerning adjustments
	
	glyphStyles[1] = NewLayoutStyle(
		(char *) "\pTimes Roman",	// the gxFontName
		ff(30),						// the text size
		0,							// no text attributes
		&controls,					// our run controls we initialized
		nil,						// no run features
		0,							// so the run feature count is zero
		&overrides);				// and our style run overrides (kerning adjustments)
	
// For the third style, we reset the run controls and kerning adjustments, but now
// set the glyph substitutions talked about earlier.

	controls.crossStreamShift = 0;							// default

	overrides.kerningAdjustments = nil;						// default
	overrides.kerningAdjustmentsCount = 0;					// default

	overrides.glyphSubstitutions = substitutions;			// our glyph subs array
	overrides.glyphSubstitutionsCount = nSubstitutions;		// our glyph subs count
	
	
// Finally, create the third style -- 60 pt. Times Roman with glyph substitutions

	glyphStyles[2] = NewLayoutStyle(
		(char *) "\pTimes Roman",	// the font name
		ff(60),						// the text size
		0,							// no text attributes
		&controls,					// our run controls (default again)
		nil,						// no run features
		0,							// so the run feature count is zero
		&overrides);				// and our style run overrides (glyph subs)

// Now we can build the layout and add it to our picture shape
	
	layout = GXNewLayout(
		3,							// count of text runs
		textLengths,				// array of lengths of each run
		(void *) textRuns,			// array of pointers to the text
		3,							// count of style runs
		textLengths,				// array of the byte length of each style run
		glyphStyles,				// the array of styles we've been constructing
		1,							// the number of levels in this layout
		&totalLength,				// the length of text this level applies to (all of it)
		&level0,					// array of next levels (this could be nil, too)
		&layoutOptions,				// our layout options (centered, flush 1/2, etc.)
		&posn);						// the position of the layout shape
		

// Retrieve the page shape so we can add to it.

	thePage = GetDocShape(wind);
	
// Add the layout to the window's picture shape.  AddToShape is in shape library.c.

	AddToShape(thePage, layout);
	
// We're done with the layout and styles, so get rid of them

	GXDisposeShape(layout);
	GXDisposeStyle(glyphStyles[0]);
	GXDisposeStyle(glyphStyles[1]);
	GXDisposeStyle(glyphStyles[2]);

// Invalidate the window's portRect so that everything gets updated.
	
	SetPort(wind);
	InvalRect(&wind->portRect);
}


/*------ DoCreateNew -----------------------------------------------------------------------*/
//
//	This routine is called when a window needs to be created.
//
OSErr DoCreateNew(void)
{
	OSErr		err;
	WindowPtr	wind;
	
// Create a window, attach a default gxViewPort to it, create and iInitialize our
// private data for it, and add a sample image to its page shape.

	wind = NewWindow(nil, &gWindowQDRect, gWindowTitle, true, noGrowDocProc,
	                 (WindowPtr)-1L, true, 0L);
	
	if (!wind)
		err = MemError();
	else
	{
		GXIgnoreGraphicsNotice(transform_already_set);
		SetDefaultViewPort(GXNewWindowViewPort(wind));	
		GXPopGraphicsNotice();		
		err = DoInitialization(wind);
	}
	
	CreateSampleLayoutImage(wind);
	return err;
}


/*------ DoDispose -------------------------------------------------------------------------*/
//
//	This routine is called when a window needs to be disposed of.
//
void DoDispose(wind)
WindowPtr wind;
{
	TH_Doc	doc;
	
// You should always dispose of your GX graphics objects before tossing your window.
// Why?  It's generally good form and this approach guarantees that everything is
// disposed.  If you had not disposed of everything, the call to DisposeWindow should
// dispose of the objects. If you are running the debugging version of QuickDraw GX
// with notices set, you will receive a notice that you had not disposed of everything.
// You can turn notices on in this file by setting gDebugging = TRUE (above).
	
	doc = (TH_Doc)  GetWRefCon(wind);	// Remember, this is where we stored our private data.
	
	GXDisposeShape(GetDocShape(wind)); 	// Dispose of this doc's shape.
	GXDisposeJob(GetDocJob(wind));		// Dispose of this doc's print job.
	DisposHandle((Handle) doc);			// Dispose of our private data.

   	DisposeWindow(wind);				// Dispose of the window.
}


/*------ DoIdle ----------------------------------------------------------------------------*/
//
//	This routine is called to do things while we're idling through the event loop.
//
void DoIdle(WindowPtr wind)
{
#pragma unused (wind)
}

/*------ DoTearDown ------------------------------------------------------------------------*/
//
//	This routine is called just before we quit to remove anything persistent that might have
//  been setup by DoSetup().
//
void DoTeardown(void)
{
}

