/*
	Hebrew Layout Sample.c
	
	This file contains a sample of a layout shape using Hebrew text.  It
	requires the Ramat Gan font if you don't want to see a lot of strange characters.
	
	1992 - 1993  Apple Computer, Inc.
	All rights reserved.
	
	Updated by Matt Deatherage.
	Originally by Eric Mader, after Mike Fairman, Oliver Steele, et. al.
*/

#include <StdIO.h>

#include <resources.h>
#include <events.h>
#include <memory.h>
#include <windows.h>
#include <toolutils.h>

#include "graphics routines.h"
#include "graphics toolbox.h"
#include "layout routines.h"
#include "graphics libraries.h"
#include "layout library.h"
#include "graphics debugging.h"
#include "PrintingMessages.h"
#include "PrintingManager.h"

#include "B3 QuickDraw GX shell.h"
#include "Hebrew Layout Sample.h"


/*------ Global variables for the shell ----------------------------------------------------*/

// Set up the title and size of the window

Str255 		gWindowTitle;
Rect 		gWindowQDRect;

// If gDebugging = TRUE, graphics library errors and notices will be posted.  This
// functionality will only work with the "debugging" version of QuickDraw GX.
// If the debugging version is not installed, nothing bad will happen, but these
// functions will not work. 

Boolean		gDebugging = true;

// Set  "gGiveMeValidation" to TRUE if you want receive run-time validation.

Boolean		gGiveMeValidation = true;


// gGraphicsHeapSize sets the size of the graphics heap created by calling the
// GXNewGraphicsClient routine in main () within B3 QuickDraw GX shell.c.  You can determine
// the amount of graphics heap required by using GraphicsBug.  I'm giving it 600K,
// since we need abunch if we have several windows open.

long		gGraphicsHeapSize = 600;


/*------ DoSetup ---------------------------------------------------------------------------*/
//
//	Here's where we initialize any global variables our application needs.  We have two of
//	them -- gWindowTitle and gWindowQDRect.  We obtain both of them from our resource fork,
//	so we don't have to recompile to change such things for international markets, etc.

void DoSetup(void)
{

	Handle	windRectHandle;

// Get the title and rectangle for our window.  These are in our resource fork -- the
// title as a string in 'STR#' resource number 'rWindTitleString' and the rectangle as
// 'RECT' resource number 'rWindRectangle'.  These are all defined in the header file
// "B3 QuickDraw GX shell.h".

	GetIndString(gWindowTitle, rWindTitleString, rWindTitleStringIndex);
	windRectHandle = GetResource('RECT',rWindRectangle);
	gWindowQDRect = *(RectPtr)*windRectHandle;
	ReleaseResource(windRectHandle);
}

/*------ DoInitialization ------------------------------------------------------------------*/
//
//	In this function we create and initialize the the private document structure for
//  a new window.  This structure contains the print job and the shape which is drawn
//  in the window.  We store this data in a handle and hang it off the window's refCon
//  field for easy retrieval.  By doing this, rather than using globals, we can create
//  many windows containing unique print jobs and shapes. 

OSErr DoInitialization(wind)
WindowPtr wind;
{
	OSErr	err = noErr;
	gxJob	docJob;
	gxShape	docPage;
	TH_Doc	windDoc;


// Create the page shape. We set the unique items attribute to make sure that each item
// added to the picture has a unique reference. If this attribute was not set, we would
// not see all copies of anything we add to the shape multiple times -- we'd just see
// the last version added.		

	docPage = GXNewShape(gxPictureType);
	GXSetShapeAttributes(docPage, gxUniqueItemsShape);
	
	
// Create a print job for this document.  This will be the same as the system default until
// the user goes through the dialogs for Page Setup or Print

	err = GXNewJob(&docJob);
	
	
// If there are no errors, create a handle the size of our document structure and store
// the print job and page shape in it.  Store the handle in the window's refCon field so
// that we can get at it.  (Note that the utility routines "GetDocJob" and "GetDocShape"
// can be used to do this easily.

	if (!err)
	{
		windDoc = (TH_Doc) NewHandleClear(sizeof(T_Doc));

		if (!windDoc)
			err = MemError();
		else
		{
			(*windDoc)->docJob = docJob;
			(*windDoc)->docPage = docPage;
			((WindowPeek) wind)->refCon = (long) windDoc;
		}

// Now install our application override for PrintingEvent so that we can
// support the new movable-modal printing dialog boxes.

		GXInstallApplicationOverride(docJob, gxPrintingEvent, MyPrintingEventOverride);

	}

	return err;
}


/*------ DoDraw ----------------------------------------------------------------------------*/
//
//	Draw the contents of the window.
// 
void DoDraw(wind)
WindowPtr wind;
{
	 GXDrawShape (GetDocShape(wind));
}


/*------ StrLength ------------------------------------------------------------------------*/
//
//	This function takes the length of a C-style string quickly without requiring an ANSI
//  library to be linked in.
//
static short StrLength(char *s)
{
	short len;

	for (len = 0; *s++ != 0; len++) ;
	return len;
}


/*------ CreateSampleLayoutImage -----------------------------------------------------------*/
//
//	This function creates  primitive shapes and adds them to the window's page shape.
//
void CreateSampleLayoutImage(WindowPtr wind)
{

	gxShape layout;						// the layout shape we build
	gxShape highlight;					// the highlight shape for the layout shape
	gxShape thePage;					// this window's document shape
	gxRunControls runControls;			// run controls for the layout shape
	gxLayoutOptions layoutOptions;		// options for the layout shape
	gxStyle timesStyle;					// a style record for a Times-based style
	gxStyle helveticaStyle;				// ditto for Helvetica
	gxStyle ramatganStyle; 				// ditto for Ramat Gan
	gxStyle textStyles[5];				// an array of text style for our text runs
	
// These are the five text runs for this layout shape, in pieces because they're
// in different languages
	
	char *text1 = "He said ";
	
// The following is "Macintosh" in Hebrew: 
// meem, kouf, yod,  noon, teth, vav,  shin
// (This assumes the standard Hebrew character set for Macintosh, by the way)
	
	static char text2[] = {0xEE, 0xF7, 0xE9, 0xF0, 0xE8, 0xE5, 0xF9, 0};
	
	char *text3 = " (Macintosh) ";

// The following is "Hebrew" in Hebrew: 
// beth, ayin, beth, rech, yoh,  tav
// Same character set assumptions.
	
	static char text4[] =	{0xE1, 0xF2, 0xE1, 0xF8, 0xE9, 0xFA, 0};

	char *text5 = 		" to me.";

	char *textRuns[5];					// an array of pointers to text runs

	short textLengths[5];				// an array of the lengths of each run
	short totalLength;					// the total length of the layout's text
	static short levelRunLengths[3];	// an array of the lengths of each level
	static short levels[3] = {0, 1, 0};	// the level numbers for each level
	
	gxPoint posn;						// the position of the layout shape


// First, initialize the textRuns array to point to the five text runs
	
	textRuns[0] = text1;
	textRuns[1] = text2;
	textRuns[2] = text3;
	textRuns[3] = text4;
	textRuns[4] = text5;
	
	
// Next, initialize the textLengths and levelRunLengths arrays.  StrLength() is
// in this file.

	textLengths[0] = StrLength (text1);
	textLengths[1] = StrLength (text2);
	textLengths[2] = StrLength (text3);
	textLengths[3] = StrLength (text4);
	textLengths[4] = StrLength (text5);
	
// Our layout has three levels in it -- level 0 for the English runs at each end,
// and level 1 for the Hebrew text in the middle.  We need the byte-count for each
// level's text run in the levelRunLengths array.

	levelRunLengths[0] = textLengths[0];
	levelRunLengths[1] = textLengths[1] + textLengths[2] + textLengths[3];
	levelRunLengths[2] = textLengths[4];

	
// totalLength is the length of all the text.

	totalLength = levelRunLengths[0] + levelRunLengths[1] + levelRunLengths[2];
	

// make default gxLayoutOptions and gxRunControls structures

	InitializeLayoutOptions (&layoutOptions);
	InitializeRunControls (&runControls);
	
	
// Position the layout half way down the left edge of the window. Set 
// the layout's width to the window's width and set the flushness to 1/2, this
// will cause the layout to center in the window.  Note that gWindowQDRect is
// not in fixed coordinates, so we have to make it fixed to use it.

	layoutOptions.width = ff(gWindowQDRect.right - gWindowQDRect.left);
	layoutOptions.flush = fract1/2;

	posn.x = 0;
	posn.y = ff((gWindowQDRect.bottom - gWindowQDRect.top) / 2);
	

// Create the styles we'll use.  The helveticaStyle is 30-point Helvetica.
// NewLayoutStyle is a library routine found in layout library.c.  But first,
// this important message:

// When I was converting this sample to actually print, I found this strange
// problem that the highlight shape, while appearing correctly on the screen,
// was always too far to the right when printed -- sometimes overlapping parts
// of glyphs that shouldn't be highlighted.  An investigation was launched.
//
// It turns out that highlight shapes are pretty much always 72 DPI, because
// they're polygons calculated from the metrics of the layout shape when the
// highlight shape is created.  If you then scale the layout shape by
// transforming it to 300 DPI or so (like a printer driver would do), the
// highlight shape has the same geometry while hinting of the glyphs may have
// changed the way the layout shape looks.  They've moved a little bit, perhaps,
// and therefore the highlight shape isn't in the right place anymore.
//
// If this is for your on-screen work, you can recreate the highlight shape in
// a zoomed viewPort, but you don't get to make the printer driver do that for
// you.  So, instead, for each style run we use, we add the gxNoMetricsGridText
// attribute.  This prevents QuickDraw GX from using hinting, so the glyphs always
// scale the same way the highlight shape does, and all is well.
//
// (At first, I tried to fix this by adding gxNoMetricsGridShape to the layout
// _shape's_ attributes, but that's not the right idea.  The attributes in the
// shape's style affect any part of the layout that doesn't have a style of its
// own, including any parts of the text for which the entry in the style array
// is nil.  Since we're actually applying styles over the layout here, the
// gxNoMetricsGridText must be set in those styles, not in the shape's style.)
//
// This does mean that your glyphs won't get hinting at low resolutions and
// small point sizes where it might be desirable -- but it's really only a problem
// if you're trying to print a highlight shape, which doesn't happen very often.
// In fact, I can't really think of a legitimate reason to want to do it, except
// maybe in sample code.
	
	helveticaStyle = NewLayoutStyle(
		(char *) "\pHelvetica",				// the gxFontName for the style
		ff(30),								// the text size in fixed point
		gxNoMetricsGridText,				// gxTextAttributes
		&runControls,						// run controls (our default ones)
		nil,								// run features (none for this style)
		0,									// count of run features
		nil);								// style run overrides (none right now)

// The second style is ramatganStyle -- Ramat Gan (Hebrew), 34 points.

	ramatganStyle = NewLayoutStyle(
		(char *) "\pRamatgan",				// gxFontName
		ff(34),								// text size (fixed point)
		gxNoMetricsGridText,				// gxTextAttributes
		&runControls,						// run controls (our default ones)
		nil, 								// run features (none)
		0, 									// count of run features (none)
		nil);								// style run overrides (none)
	
// The final style is 30-point Times Roman.

	timesStyle = NewLayoutStyle(
		(char *) "\pTimes Roman", 			// gxFontName
		ff(30), 							// text size (fixed point)
		gxNoMetricsGridText,				// gxTextAttributes
		&runControls, 						// run controls (our default ones)
		nil, 								// run features (none)
		0, 									// count of run features (none)
		nil);								// style run overrides (none)
	

// Initialize the textStyles array so that each of the five runs has the right style

	textStyles[0] = helveticaStyle;
	textStyles[1] = ramatganStyle;
	textStyles[2] = timesStyle;
	textStyles[3] = ramatganStyle;
	textStyles[4] = helveticaStyle;
	

// Setup complete!  Build the layout.

	layout = GXNewLayout(
		5,								// count of text runs
		textLengths,					// array of lengths of each run
		(void *) textRuns,				// array of pointers to the text
		5,								// count of style runs
		textLengths,					// array of the byte lengths of each run
		textStyles,						// array of the styles
		3,								// number of levels in the layout
		levelRunLengths,				// array of the lengths of each level
		levels,							// array of the levels
		&layoutOptions,					// options (default)
		&posn);							// position of the layout shape

	
// Retrieve the page shape so we can add to it.

	thePage = GetDocShape(wind);
	
// Add the layout to the window's picture shape.  AddToShape is in shape library.c.

	AddToShape(thePage, layout);

// Make a highlight shape to highlight the layout from character 5 to character 20.
// This will go from the level 0 English text into the level 2 English text
// which is nested in the level 1 Hebrew text, and will produce a highlight shape
// consisting of three disjoint rectangles because the Hebrew text is displayed
// right-to-left.  SetShapeCommonTransfer is in transferMode library.c, and
// SetShapeCommonColor is in color library.c.

	highlight = GXGetLayoutHighlight(
		layout,							// the layout shape to get the highlight from
		5, 20, 							// starting and ending byte offsets
		gxHighlightAverageAngle, 		// highlight type -- use the average angle
		nil);							// a shape to use if we have one around (we don't)
	
	SetShapeCommonTransfer (highlight, gxHighlightMode);
	SetShapeCommonColor (highlight, gxWhite);

	
	AddToShape(thePage, highlight);
	

// Dispose of what we allocated

	GXDisposeShape(highlight);
	GXDisposeShape(layout);
	GXDisposeStyle(helveticaStyle);
	GXDisposeStyle(ramatganStyle);
	GXDisposeStyle(timesStyle);
	

// Invalidate the window's portRect so that everything gets updated.
	
	SetPort(wind);
	InvalRect(&wind->portRect);
}


/*------ DoCreateNew -----------------------------------------------------------------------*/
//
//	This routine is called when a window needs to be created.
//
OSErr DoCreateNew(void)
{
	OSErr		err;
	WindowPtr	wind;
	
// Create a window, attach a default gxViewPort to it, create and iInitialize our
// private data for it, and add a sample image to its page shape.

	wind = NewWindow(nil, &gWindowQDRect, gWindowTitle, true, noGrowDocProc,
	                 (WindowPtr)-1L, true, 0L);
	
	if (!wind)
		err = MemError();
	else
	{
		GXIgnoreGraphicsNotice(transform_already_set);
		SetDefaultViewPort(GXNewWindowViewPort(wind));			
		GXPopGraphicsNotice();
		err = DoInitialization(wind);
	}
	
	CreateSampleLayoutImage(wind);
	return err;
}


/*------ DoDispose -------------------------------------------------------------------------*/
//
//	This routine is called when a window needs to be disposed of.
//
void DoDispose(wind)
WindowPtr wind;
{
	TH_Doc	doc;
	
// You should always dispose of your GX graphics objects before tossing your window.
// Why?  It's generally good form and this approach guarantees that everything is
// disposed.  If you had not disposed of everything, the call to DisposeWindow should
// dispose of the objects. If you are running the debugging version of QuickDraw GX
// with notices set, you will receive a notice that you had not disposed of everything.
// You can turn notices on in this file by setting gDebugging = TRUE (above).
	
	doc = (TH_Doc)  GetWRefCon(wind);	// Remember, this is where we stored our private data.
	
	GXDisposeShape(GetDocShape(wind)); 	// Dispose of this doc's shape.
	GXDisposeJob(GetDocJob(wind));		// Dispose of this doc's print job.
	DisposHandle((Handle) doc);			// Dispose of our private data.

   	DisposeWindow(wind);				// Dispose of the window.
}


/*------ DoIdle ----------------------------------------------------------------------------*/
//
//	This routine is called to do things while we're idling through the event loop.
//
void DoIdle(WindowPtr wind)
{
#pragma unused (wind)
}

/*------ DoTearDown ------------------------------------------------------------------------*/
//
//	This routine is called just before we quit to remove anything persistent that might have
//  been setup by DoSetup().
//
void DoTeardown(void)
{
}

