/*
	Application: Multi-gxColor Layout
	
	Description: 
		Displays a 6 run multi-colored layout using text faces.

		This application makes one 6 run layout and copies it 5 times to produce
		6 different colored layouts.  Using text faces, one run in each layout
		is made visible by setting the gxFaceLayer's outlineFill to gxSolidFill while
		the other "invisible" runs have an outlineFill of gxNoFill.

		Having a copy of the layout for each gxColor is probablly a cumbersome way
		of implementing multi-colored layouts.  There is a synchronization issue
		in that any operation on what appears to be one layout must actually be
		done on all the layouts.

		If the symbol LAYOUTS_IN_PICTURE is true all the layouts are added to a
		picture to contain them in one gxShape

		Known Bugs:
		* Depending on when the layout is drawn can give give different results
		for the colored layouts.  If each layout is drawn just after the call to
		SetLayoutRunColor the results are correct.  However, if all the layouts
		are drawn after their gxColor is set then the last run of each layout appears
		as the "visible" run in the gxShape's gxColor.

		* Likewise, adding all the layouts to a picture and drawing this picture
		results in the behavior described above.

	QuickDraw GX Version: 1.0b2

	Other Source Files Required:
		"color library.c"
		"transform library.c"

	Revision History: 
		9/18/92		Robert Dierkes			Created
		3/31/93		Pete "Luke" Alexander 	Updated to run under v1.0a5
		4/23/93		Pete "Luke" Alexander 	Updated to run under v1.0b1
											GXResetMapping was the only call changed
		9/09/93		Matt Deatherage			Updated for 1.0b2.  Added MPW makefile.
											Changed Math Routines calls back to
											not having "GX" prefixes.
*/


/*________ Includes ________________________________________________________________*/

#include <Memory.h>
#include <Fonts.h>
#include <Windows.h>
#include <Desk.h>
#include "graphics libraries.h"
#include "graphics toolbox.h"
#include "layout routines.h"
#include "font routines.h"


/*________ ProtoTypes ______________________________________________________________*/

Boolean AppSetup (WindowPtr *pWindow, gxViewPort *pWindViewPort);
long StringLength (char *s);
Boolean SetLayoutRunColor (gxShape layout, long visibleRun, commonColor colour);
gxStyle NewStyleNamedFontAndSize (unsigned char* name, fixed textSize);


#define		kWindowWidth	512
#define		kWindowHeight	250
#define		kWindowOffset	30

#define		kFontName		"\pHoefler Italic"
#define		kRunCount		6
#define		kTextSize		IntToFixed (36)
#define		kFIrstColor		red

#define		LAYOUTS_IN_PICTURE	true


void main ()
{
	WindowPtr	window;
	gxViewPort	gWindViewPort;
	long		i;
	char		*inText [kRunCount] = {"Red", "Green", "Blue", "Cyan", "Magenta", "Yellow"};
	gxStyle		inRunStyles [kRunCount];
	short		inRunLengths [kRunCount];
	gxPoint		position = {IntToFixed (15), IntToFixed (120)};
	gxShape 		colorLayout [kRunCount];
	gxShape 		layoutPicture;

	if (! AppSetup (&window, &gWindViewPort))
		return;

	/* Make the layout */
	for (i = 0; i < kRunCount; i++)
	{
		inRunLengths [i] = StringLength ((char *) inText [i]);
		inRunStyles [i]  = NewStyleNamedFontAndSize (kFontName, kTextSize);
	}

	colorLayout [0] = GXNewLayout (kRunCount,			/* textRunCount		*/
								inRunLengths,		/* textRunLengths[]	*/
								(void *) inText,	/* *text[]			*/
								kRunCount,			/* styleRunCount	*/
								inRunLengths,		/* styleRunLengths[]*/
								inRunStyles,		/* styles[]			*/
								0,					/* levelRunCount	*/
								nil,				/* levelRunLengths[]*/
								nil,				/* levels[]			*/
								nil,				/* *gxLayoutOptions	*/
								&position );		/* *position		*/

	/* Copy the first layout for all the other colors */
	for (i = 1; i < kRunCount; i++)
		colorLayout [i] = CopyShape (colorLayout [0]);

	if (LAYOUTS_IN_PICTURE)
	{
		layoutPicture = GXNewShape (gxPictureType);
		GXSetShapeAttributes (layoutPicture, gxUniqueItemsShape);
	}

	/* Set the gxColor of each run and add a copy of the layout to the picture */
	for (i = 0; i < kRunCount; i++)
	{
		if (LAYOUTS_IN_PICTURE)
		{
			if (SetLayoutRunColor (colorLayout [i], i, kFIrstColor + i))
				GXSetPictureParts (layoutPicture, 0, 0, 1, &colorLayout [i], nil, nil, nil);
		}
		else
		{
			SetLayoutRunColor (colorLayout [i], i, kFIrstColor + i);
			GXDrawShape (colorLayout [i]);
		}
	}

	if (LAYOUTS_IN_PICTURE)
		GXDrawShape (layoutPicture);
	/** If all the layouts are drawn here the last run	**
	 **	of each layout will appear as the colored run.	**
	else
		for (i = 0; i < kRunCount; i++)
			GXDrawShape (colorLayout [i]);
	**/

	while (! Button ())
		SystemTask ();

	if (LAYOUTS_IN_PICTURE)
		GXDisposeShape (layoutPicture);
	for (i = 0; i < kRunCount; i++)
		GXDisposeShape (colorLayout [i]);
	for (i = 0; i < kRunCount; i++)
		GXDisposeStyle (inRunStyles [i]);
	DisposeCommonColors ();
	GXDisposeViewPort (gWindViewPort);
	DisposeWindow (window);
}


Boolean SetLayoutRunColor (gxShape layout, long visibleRun, commonColor colour)
{
	long		styleCount;
	gxStyle		*pStyles;
	long		index;
	gxTextFace	face;
	gxFaceLayer	*pLayer;

	if (GXGetShapeType (layout) != gxLayoutType)
		return (false);

	GXGetLayout (	layout,				/* layout			*/
				nil,				/* *text			*/
			    &styleCount,		/* *styleRunCount	*/
				nil,				/* styleRunLengths[]*/
				nil,				/* styles[]			*/
				nil,				/* *levelRunCount	*/
				nil,				/* levels[]			*/
				nil,				/* levelRunLengths[]*/
				nil,				/* *gxLayoutOptions	*/
				nil);				/* *position		*/


	if ((pStyles = (gxStyle *) NewPtr (styleCount * sizeof (gxStyle))) == nil)
	{
		DebugStr ("\pNewPtr for layout styles failed.");
		return (false);
	}

	/* Get the layout's styles */
	GXGetLayout (	layout,				/* layout			*/
				nil,				/* *text			*/
				nil,				/* *styleRunCount	*/
				nil,				/* styleRunLengths[]*/
				pStyles,			/* styles[]			*/
				nil,				/* *levelRunCount	*/
				nil,				/* levels[]			*/
				nil,				/* levelRunLengths[]*/
				nil,				/* *gxLayoutOptions	*/
				nil);				/* *position		*/

	/* Set text face for each gxStyle */
	face.faceLayers = 1;
	ResetMapping (&face.advanceMapping);

	pLayer = face.faceLayer;
	pLayer->flags = 0;
	pLayer->outlineStyle = nil;
	pLayer->outlineTransform = nil;
	pLayer->boldOutset.x =
	pLayer->boldOutset.y = 0;

	/* Only the visibleRun will have gxSolidFill to make it visible */
	for (index = 0; index < styleCount; index++)
	{
		if (pStyles [index] != nil)
		{
			pLayer->outlineFill = (index == visibleRun) ? gxSolidFill : gxNoFill;
			GXSetStyleFace (pStyles [index], &face);
		}
	}

	DisposPtr ((Ptr) pStyles);

	SetShapeCommonColor (layout, colour);

	return (true);
}


Boolean AppSetup (WindowPtr *pWindow, gxViewPort *pWindViewPort)
{
	Rect		bounds;
	GDHandle	hGDevice;

	if (!pWindow  ||  !pWindViewPort)
		return (false);

	InitGraf (&qd.thePort);
	InitFonts ();
	InitWindows ();

	SetRect (&bounds, -32767, -32767, 32767, 32767);
	hGDevice = GetMaxDevice (&bounds);

	bounds = (**hGDevice).gdRect;
	bounds.left  += kWindowOffset;
	bounds.right  = bounds.left + kWindowWidth;
	bounds.top   += kWindowOffset + 20;
	bounds.bottom = bounds.top + kWindowHeight;
	*pWindow = NewWindow (nil, &bounds, "\pLayouts in Color", true, documentProc, (WindowPtr) -1, false, 0);

	SetDefaultViewPort (*pWindViewPort = GXNewWindowViewPort (*pWindow));

	return (true);
}


long StringLength (char *s)
{
	const char *p = s;

	while (*p != 0)
		p++;
	return (p - s);
}


gxStyle NewStyleNamedFontAndSize (unsigned char* name, fixed textSize)
{
	gxStyle		simpleStyle;
	gxFont		fontID;

	simpleStyle = GXNewStyle ();

	/* Doesn't depend on FindPNameFont in "gxFont libarary.c" */
	fontID = GXGetDefaultFont();
	GXFindFonts (0, gxFullFontName, gxMacintoshPlatform, gxRomanScript, gxEnglishLanguage, name[0], (unsigned char*) &name[1], 1, 1, &fontID);
	if (fontID != GXGetStyleFont (simpleStyle))
		GXSetStyleFont (simpleStyle, fontID);

	GXSetStyleTextSize (simpleStyle, textSize);

	return (simpleStyle);
}
